@@ -31,26 +31,105 @@
 #include "grepmap.h"
 
 
-/* Flags specifying what to match */
-typedef enum {
-	IEEE1394_MATCH_VENDOR        = (1 << 0),
-	IEEE1394_MATCH_MODEL         = (1 << 1),
-	IEEE1394_MATCH_SPECIFIER     = (1 << 2),
-	IEEE1394_MATCH_VERSION       = (1 << 3)
-} Ieee1394MatchFlags;
+/**
+ * ieee1394map_parse_args:
+ * @argc: number of arguments,
+ * @argv: arguments array,
+ * @dev: device structure to fill.
+ *
+ * Parses the command-line arguments to fill a device structure.
+ *
+ * Returns: 0 on success, non-zero on failure.
+ **/
+int
+ieee1394map_parse_args (int             argc,
+			char           *argv[],
+			Ieee1394Device *dev)
+{
+	int ret = 0;
 
-/* IEEE1394 Device structure */
-typedef struct {
-	Ieee1394MatchFlags match_flags;
-	unsigned int       vendor, model, specifier, version;
-} Ieee1394Device;
+	if (argc < 4) {
+		fprintf (stderr, "%s: %s\n", program_name,
+			 _("insufficient arguments"));
+		suggest_help ();
+		return 2;
+	}
 
+	ret |= parse_hex (argv[0], &dev->vendor);
+	ret |= parse_hex (argv[1], &dev->model);
+	ret |= parse_hex (argv[2], &dev->specifier);
+	ret |= parse_hex (argv[3], &dev->version);
+	if (ret) {
+		fprintf (stderr, "%s: %s\n", program_name,
+			 _("arguments must be in hexadecimal format"));
+		suggest_help ();
+		return 2;
+	}
+
+	if (argc > 4)
+		fprintf (stderr, "%s: %s\n", program_name,
+			 _("some arguments ignored"));
+
+	return 0;
+}
+
+/**
+ * ieee1394map_udev_event:
+ * @environ: environment array,
+ * @dev: device structure to fill.
+ *
+ * Fills the device structure by reading a udev event from the environment.
+ *
+ * Returns: 0 on success, non-zero on failure.
+ **/
+int
+ieee1394map_udev_event (char           **environ,
+			Ieee1394Device  *dev)
+{
+	char **envvar;
+	int    ret = 0;
+
+	for (envvar = environ; *envvar; envvar++) {
+		char *key, *value;
+
+		key = strdup (*envvar);
+		value = strchr (key, '=');
+		if (! value)
+			goto next;
+		*(value++) = 0;
+
+		if (! strcmp (key, "VENDOR_ID")) {
+			ret |= parse_hex (value, &dev->vendor);
+
+		} else if (! strcmp (key, "MODEL_ID")) {
+			ret |= parse_hex (value, &dev->model);
+
+		} else if (! strcmp (key, "SPECIFIER_ID")) {
+			ret |= parse_hex (value, &dev->specifier);
+
+		} else if (! strcmp (key, "VERSION")) {
+			ret |= parse_hex (value, &dev->version);
+
+		}
+
+	next:
+		free (key);
+	}
+
+	if (ret) {
+		fprintf (stderr, "%s: %s\n", program_name,
+			 _("udev event not in expected format"));
+		return 2;
+	}
+
+	return 0;
+}
 
 /**
  * ieee1394map_modules:
- * @mapf: open map file to read from.
- * @file: filename or map file, or '-' for standard input.
- * @args: arguments (vendor, specifier, version)
+ * @mapf: open map file to read from,
+ * @file: filename or map file, or '-' for standard input,
+ * @dev: device to look for.
  *
  * Handles the modules.ieee1394map file looking for a matching entry for
  * the device given, printing the module name to stdout if found.
@@ -58,23 +137,12 @@ typedef struct {
  * Returns: 0 on success, non-zero on failure.
  **/
 int
-ieee1394map_modules (FILE       *mapf,
-		const char *file,
-		char       *argv[])
+ieee1394map_modules (FILE           *mapf,
+		     const char     *file,
+		     Ieee1394Device *dev)
 {
-	Ieee1394Device  dev;
-	char      *line;
-	int        lineno = 0, ret = 0;
-
-	ret |= parse_hex (argv[0], &dev.vendor);
-	ret |= parse_hex (argv[1], &dev.specifier);
-	ret |= parse_hex (argv[2], &dev.version);
-	if (ret) {
-		fprintf (stderr, "%s: %s\n", program_name,
-			 _("arguments must be in hexadecimal format"));
-		suggest_help ();
-		return 2;
-	}
+	char *line;
+	int   lineno = 0, ret = 0;
 
 	ret = 1;
 	while ((line = fgets_alloc (mapf)) != NULL) {
@@ -95,21 +163,19 @@ ieee1394map_modules (FILE       *mapf,
 		}
 
 		if (FLAG_SET(map.match_flags, IEEE1394_MATCH_VENDOR)
-		    && (map.vendor != dev.vendor))
+		    && (map.vendor != dev->vendor))
 			continue;
 
-		/* NOTE: hotplug doesn't seem to have the model id handy,
-		 * that's why we don't look for it here.  Also hotplug
-		 * never checks that the match_flags require a model to
-		 * be matched -- bug for bug compatible again.
-		 */
+		if (FLAG_SET(map.match_flags, IEEE1394_MATCH_MODEL)
+		    && (map.model != dev->model))
+			continue;
 
 		if (FLAG_SET(map.match_flags, IEEE1394_MATCH_SPECIFIER)
-		    && (map.specifier != dev.specifier))
+		    && (map.specifier != dev->specifier))
 			continue;
 
 		if (FLAG_SET(map.match_flags, IEEE1394_MATCH_VERSION)
-		    && (map.version != dev.version))
+		    && (map.version != dev->version))
 			continue;
 
 		line[strcspn (line, " \t\r\n")] = '\0';
