@@ -31,23 +31,123 @@
 #include "grepmap.h"
 
 
-/* The "any device" id */
-#define PCI_ANY 0xffffffff
+/**
+ * pcimap_parse_args:
+ * @argc: number of arguments,
+ * @argv: arguments array,
+ * @dev: device structure to fill.
+ *
+ * Parses the command-line arguments to fill a device structure.
+ *
+ * Returns: 0 on success, non-zero on failure.
+ **/
+int
+pcimap_parse_args (int        argc,
+		   char      *argv[],
+		   PciDevice *dev)
+{
+	int ret = 0;
 
+	if (argc < 5) {
+		fprintf (stderr, "%s: %s\n", program_name,
+			 _("insufficient arguments"));
+		suggest_help ();
+		return 2;
+	}
 
-/* PCI Device structure */
-typedef struct {
-	unsigned int id_vendor, id_device;
-	unsigned int subid_vendor, subid_device;
-	unsigned int class, class_mask;
-} PciDevice;
+	ret |= parse_hex (argv[0], &dev->id_vendor);
+	ret |= parse_hex (argv[1], &dev->id_device);
+	ret |= parse_hex (argv[2], &dev->subid_vendor);
+	ret |= parse_hex (argv[3], &dev->subid_device);
+	ret |= parse_hex (argv[4], &dev->class);
+	if (ret) {
+		fprintf (stderr, "%s: %s\n", program_name,
+			 _("arguments must be in hexadecimal format"));
+		suggest_help ();
+		return 2;
+	}
 
+	if (argc > 5)
+		fprintf (stderr, "%s: %s\n", program_name,
+			 _("some arguments ignored"));
+
+	return 0;
+}
+
+/**
+ * pcimap_udev_event:
+ * @environ: environment array,
+ * @dev: device structure to fill.
+ *
+ * Fills the device structure by reading a udev event from the environment.
+ *
+ * Returns: 0 on success, non-zero on failure.
+ **/
+int
+pcimap_udev_event (char      **environ,
+		   PciDevice  *dev)
+{
+	char **envvar;
+	int    ret = 0;
+
+	for (envvar = environ; *envvar; envvar++) {
+		char *key, *value;
+
+		key = strdup (*envvar);
+		value = strchr (key, '=');
+		if (! value)
+			goto next;
+		*(value++) = 0;
+
+		if (! strcmp (key, "PCI_ID")) {
+			char *ptr;
+
+			ptr = strchr (value, ':');
+			if (! ptr) {
+				ret = 1;
+				goto next;
+			}
+			*(ptr++) = 0;
+
+			ret |= parse_hex (value, &dev->id_vendor);
+			ret |= parse_hex (ptr, &dev->id_device);
+
+		} else if (! strcmp (key, "PCI_SUBSYS_ID")) {
+			char *ptr;
+
+			ptr = strchr (value, ':');
+			if (! ptr) {
+				ret = 1;
+				goto next;
+			}
+			*(ptr++) = 0;
+
+			ret |= parse_hex (value, &dev->subid_vendor);
+			ret |= parse_hex (ptr, &dev->subid_device);
+
+		} else if (! strcmp (key, "PCI_CLASS")) {
+			ret |= parse_hex (value, &dev->class);
+
+		}
+
+	next:
+		free (key);
+	}
+
+	if (ret) {
+		fprintf (stderr, "%s: %s\n", program_name,
+			 _("udev event not in expected format"));
+		return 2;
+	}
+
+	return 0;
+}
 
 /**
  * pcimap_modules:
- * @mapf: open map file to read from.
- * @file: filename or map file, or '-' for standard input.
- * @args: arguments (vendor, device, subvendor, subdevice, class)
+ * @mapf: open map file to read from,
+ * @file: filename or map file, or '-' for standard input,
+ * @dev: device to look for.
  *
  * Handles the modules.pcimap file looking for a matching entry for
  * the device given, printing the module name to stdout if found.
@@ -57,23 +157,10 @@ typedef struct {
 int
 pcimap_modules (FILE       *mapf,
 		const char *file,
-		char       *argv[])
+		PciDevice  *dev)
 {
-	PciDevice  dev;
-	char      *line;
-	int        lineno = 0, ret = 0;
-
-	ret |= parse_hex (argv[0], &dev.id_vendor);
-	ret |= parse_hex (argv[1], &dev.id_device);
-	ret |= parse_hex (argv[2], &dev.subid_vendor);
-	ret |= parse_hex (argv[3], &dev.subid_device);
-	ret |= parse_hex (argv[4], &dev.class);
-	if (ret) {
-		fprintf (stderr, "%s: %s\n", program_name,
-			 _("arguments must be in hexadecimal format"));
-		suggest_help ();
-		return 2;
-	}
+	char *line;
+	int   lineno = 0, ret = 0;
 
 	ret = 1;
 	while ((line = fgets_alloc (mapf)) != NULL) {
@@ -94,23 +181,23 @@ pcimap_modules (FILE       *mapf,
 		}
 
 		if ((map.id_vendor != PCI_ANY)
-		    && (map.id_vendor != dev.id_vendor))
+		    && (map.id_vendor != dev->id_vendor))
 			continue;
 
 		if ((map.id_device != PCI_ANY)
-		    && (map.id_device != dev.id_device))
+		    && (map.id_device != dev->id_device))
 			continue;
 
 		if ((map.subid_vendor != PCI_ANY)
-		    && (map.subid_vendor != dev.subid_vendor))
+		    && (map.subid_vendor != dev->subid_vendor))
 			continue;
 
 		if ((map.subid_device != PCI_ANY)
-		    && (map.subid_device != dev.subid_device))
+		    && (map.subid_device != dev->subid_device))
 			continue;
 
 		if ((map.class & map.class_mask)
-		    != (dev.class & map.class_mask))
+		    != (dev->class & map.class_mask))
 			continue;
 
 		line[strcspn (line, " \t\r\n")] = '\0';
