@@ -31,37 +31,161 @@
 #include "grepmap.h"
 
 
-/* Flags specifying what to match */
-typedef enum {
-	USB_MATCH_VENDOR        = (1 << 0),
-	USB_MATCH_PRODUCT       = (1 << 1),
-	USB_MATCH_DEV_LO        = (1 << 2),
-	USB_MATCH_DEV_HI        = (1 << 3),
-	USB_MATCH_DEV_CLASS     = (1 << 4),
-	USB_MATCH_DEV_SUBCLASS  = (1 << 5),
-	USB_MATCH_DEV_PROTOCOL  = (1 << 6),
-	USB_MATCH_INT_CLASS     = (1 << 7),
-	USB_MATCH_INT_SUBCLASS  = (1 << 8),
-	USB_MATCH_INT_PROTOCOL  = (1 << 9)
-} UsbMatchFlags;
+/**
+ * usbmap_parse_args:
+ * @argc: number of arguments,
+ * @argv: arguments array,
+ * @dev: device structure to fill.
+ *
+ * Parses the command-line arguments to fill a device structure.
+ *
+ * Returns: 0 on success, non-zero on failure.
+ **/
+int
+usbmap_parse_args (int        argc,
+		   char      *argv[],
+		   UsbDevice *dev)
+{
+	int ret = 0;
 
-/* USB Device structure */
-typedef struct {
-	UsbMatchFlags match_flags;
-	unsigned int  vendor, product;
-	unsigned int  dev, dev_lo, dev_hi;
-	unsigned int  dev_class, dev_subclass, dev_protocol;
-	unsigned int  int_class, int_subclass, int_protocol;
-} UsbDevice;
+	if (argc < 9) {
+		fprintf (stderr, "%s: %s\n", program_name,
+			 _("insufficient arguments"));
+		suggest_help ();
+		return 2;
+	}
 
+	ret |= parse_hex (argv[0], &dev->vendor);
+	ret |= parse_hex (argv[1], &dev->product);
+	ret |= parse_hex (argv[2], &dev->dev);
+	ret |= parse_hex (argv[3], &dev->dev_class);
+	ret |= parse_hex (argv[4], &dev->dev_subclass);
+	ret |= parse_hex (argv[5], &dev->dev_protocol);
+	ret |= parse_hex (argv[6], &dev->int_class);
+	ret |= parse_hex (argv[7], &dev->int_subclass);
+	ret |= parse_hex (argv[8], &dev->int_protocol);
+	if (ret) {
+		fprintf (stderr, "%s: %s\n", program_name,
+			 _("arguments must be in hexadecimal format"));
+		suggest_help ();
+		return 2;
+	}
+
+	if (argc > 9)
+		fprintf (stderr, "%s: %s\n", program_name,
+			 _("some arguments ignored"));
+
+	return 0;
+}
+
+/**
+ * usbmap_udev_event:
+ * @environ: environment array,
+ * @dev: device structure to fill.
+ *
+ * Fills the device structure by reading a udev event from the environment.
+ *
+ * Returns: 0 on success, non-zero on failure.
+ **/
+int
+usbmap_udev_event (char      **environ,
+		   UsbDevice  *dev)
+{
+	char **envvar;
+	int    ret = 0;
+
+	for (envvar = environ; *envvar; envvar++) {
+		char *key, *value;
+
+		key = strdup (*envvar);
+		value = strchr (key, '=');
+		if (! value)
+			goto next;
+		*(value++) = 0;
+
+		if (! strcmp (key, "PRODUCT")) {
+			char *ptr1, *ptr2;
+
+			ptr1 = strchr (value, '/');
+			if (! ptr1) {
+				ret = 1;
+				goto next;
+			}
+			*(ptr1++) = 0;
+
+			ptr2 = strchr (ptr1, '/');
+			if (! ptr2) {
+				ret = 1;
+				goto next;
+			}
+			*(ptr2++) = 0;
+
+			ret |= parse_hex (value, &dev->vendor);
+			ret |= parse_hex (ptr1, &dev->product);
+			ret |= parse_hex (ptr2, &dev->dev);
+
+		} else if (! strcmp (key, "TYPE")) {
+			char *ptr1, *ptr2;
+
+			ptr1 = strchr (value, '/');
+			if (! ptr1) {
+				ret = 1;
+				goto next;
+			}
+			*(ptr1++) = 0;
+
+			ptr2 = strchr (ptr1, '/');
+			if (! ptr2) {
+				ret = 1;
+				goto next;
+			}
+			*(ptr2++) = 0;
+
+			ret |= parse_hex (value, &dev->dev_class);
+			ret |= parse_hex (ptr1, &dev->dev_subclass);
+			ret |= parse_hex (ptr2, &dev->dev_protocol);
+
+		} else if (! strcmp (key, "INTERFACE")) {
+			char *ptr1, *ptr2;
+
+			ptr1 = strchr (value, '/');
+			if (! ptr1) {
+				ret = 1;
+				goto next;
+			}
+			*(ptr1++) = 0;
+
+			ptr2 = strchr (ptr1, '/');
+			if (! ptr2) {
+				ret = 1;
+				goto next;
+			}
+			*(ptr2++) = 0;
+
+			ret |= parse_hex (value, &dev->int_class);
+			ret |= parse_hex (ptr1, &dev->int_subclass);
+			ret |= parse_hex (ptr2, &dev->int_protocol);
+
+		}
+
+	next:
+		free (key);
+	}
+
+	if (ret) {
+		fprintf (stderr, "%s: %s\n", program_name,
+			 _("udev event not in expected format"));
+		return 2;
+	}
+
+	return 0;
+}
 
 /**
  * usbmap_modules:
- * @mapf: open map file to read from.
- * @file: filename or map file, or '-' for standard input.
- * @args: arguments (vendor, product, dev,
- *                   dev_class, dev_subclass, dev_protocol,
- *                   int_class, int_subclass, int_protocol)
+ * @mapf: open map file to read from,
+ * @file: filename or map file, or '-' for standard input,
+ * @dev: device to look for.
  *
  * Handles the modules.usbmap file looking for a matching entry for
  * the device given, printing the module name to stdout if found.
@@ -71,27 +195,10 @@ typedef struct {
 int
 usbmap_modules (FILE       *mapf,
 		const char *file,
-		char       *argv[])
+		UsbDevice  *dev)
 {
-	UsbDevice  dev;
-	char      *line;
-	int        lineno = 0, ret = 0;
-
-	ret |= parse_hex (argv[0], &dev.vendor);
-	ret |= parse_hex (argv[1], &dev.product);
-	ret |= parse_hex (argv[2], &dev.dev);
-	ret |= parse_hex (argv[3], &dev.dev_class);
-	ret |= parse_hex (argv[4], &dev.dev_subclass);
-	ret |= parse_hex (argv[5], &dev.dev_protocol);
-	ret |= parse_hex (argv[6], &dev.int_class);
-	ret |= parse_hex (argv[7], &dev.int_subclass);
-	ret |= parse_hex (argv[8], &dev.int_protocol);
-	if (ret) {
-		fprintf (stderr, "%s: %s\n", program_name,
-			 _("arguments must be in hexadecimal format"));
-		suggest_help ();
-		return 2;
-	}
+	char *line;
+	int   lineno = 0, ret = 0;
 
 	ret = 1;
 	while ((line = fgets_alloc (mapf)) != NULL) {
@@ -116,43 +223,43 @@ usbmap_modules (FILE       *mapf,
 		}
 
 		if (FLAG_SET(map.match_flags, USB_MATCH_VENDOR)
-		    && (map.vendor != dev.vendor))
+		    && (map.vendor != dev->vendor))
 			continue;
 
 		if (FLAG_SET(map.match_flags, USB_MATCH_PRODUCT)
-		    && (map.product != dev.product))
+		    && (map.product != dev->product))
 			continue;
 
 		if (FLAG_SET(map.match_flags, USB_MATCH_DEV_LO)
-		    && (map.dev_lo > dev.dev))
+		    && (map.dev_lo > dev->dev))
 			continue;
 
 		if (FLAG_SET(map.match_flags, USB_MATCH_DEV_HI)
-		    && (map.dev_hi < dev.dev))
+		    && (map.dev_hi < dev->dev))
 			continue;
 
 		if (FLAG_SET(map.match_flags, USB_MATCH_DEV_CLASS)
-		    && (map.dev_class != dev.dev_class))
+		    && (map.dev_class != dev->dev_class))
 			continue;
 
 		if (FLAG_SET(map.match_flags, USB_MATCH_DEV_SUBCLASS)
-		    && (map.dev_subclass != dev.dev_subclass))
+		    && (map.dev_subclass != dev->dev_subclass))
 			continue;
 
 		if (FLAG_SET(map.match_flags, USB_MATCH_DEV_PROTOCOL)
-		    && (map.dev_protocol != dev.dev_protocol))
+		    && (map.dev_protocol != dev->dev_protocol))
 			continue;
 
 		if (FLAG_SET(map.match_flags, USB_MATCH_INT_CLASS)
-		    && (map.int_class != dev.int_class))
+		    && (map.int_class != dev->int_class))
 			continue;
 
 		if (FLAG_SET(map.match_flags, USB_MATCH_INT_SUBCLASS)
-		    && (map.int_subclass != dev.int_subclass))
+		    && (map.int_subclass != dev->int_subclass))
 			continue;
 
 		if (FLAG_SET(map.match_flags, USB_MATCH_INT_PROTOCOL)
-		    && (map.int_protocol != dev.int_protocol))
+		    && (map.int_protocol != dev->int_protocol))
 			continue;
 
 		line[strcspn (line, " \t\r\n")] = '\0';
